@@ -1,3 +1,4 @@
+use crate::js_utils::adapters::proxies::JsProxyInstanceId;
 use crate::js_utils::adapters::{JsRealmAdapter, JsRuntimeAdapter};
 use crate::js_utils::facades::{JsRuntimeFacade, JsRuntimeFacadeInner, JsValueType};
 use crate::js_utils::JsError;
@@ -209,29 +210,86 @@ impl CachedJsPromiseRef {
 impl CachedJsArrayRef {
     pub async fn js_get_array<R: JsRuntimeFacadeInner>(
         &self,
-        _rti: &R,
+        rti: &R,
     ) -> Result<Vec<JsValueFacade>, JsError> {
-        todo!()
+        let res = self
+            .cached_object
+            .with_obj(rti, |realm, arr| {
+                let mut vec: Vec<JsValueFacade> = vec![];
+                realm.js_array_traverse_mut(arr, |_index, element| {
+                    vec.push(realm.to_js_value_facade(element)?);
+                    Ok(())
+                })?;
+                Ok(vec)
+            })
+            .await?;
+        res
     }
 }
 
 impl CachedJsFunctionRef {
-    pub async fn js_invoke_function<R: JsRuntimeFacadeInner>(
+    pub fn js_invoke_function<R: JsRuntimeFacadeInner>(
         &self,
-        _rti: &R,
-        _args: Vec<JsValueFacade>,
-    ) -> Result<JsValueFacade, JsError> {
-        todo!()
+        rti: &R,
+        args: Vec<JsValueFacade>,
+    ) -> Pin<Box<dyn futures::Future<Output = Result<JsValueFacade, JsError>> + Send>> {
+        //Pin<Box<dyn futures::Future<Output = Result<JsValueFacade, JsError>>>>
+        let cached_obj_id = self.cached_object.id;
+        let realm_id = self.cached_object.realm_id.clone();
+
+        rti.js_add_rt_task_to_event_loop(move |rt| {
+            //
+            if let Some(realm) = rt.js_get_realm(realm_id.as_str()) {
+                realm.js_cache_with(cached_obj_id, move |func_adapter| {
+                    let mut adapter_args = vec![];
+                    for arg in args {
+                        adapter_args.push(realm.from_js_value_facade(arg)?);
+                    }
+
+                    let adapter_refs: Vec<&<<<<R as JsRuntimeFacadeInner>::JsRuntimeFacadeType as JsRuntimeFacade>::JsRuntimeAdapterType as JsRuntimeAdapter>::JsRealmAdapterType as JsRealmAdapter>::JsValueAdapterType> = adapter_args.iter().collect();
+
+                    let val_adapter = realm.js_function_invoke(
+                        None,
+                        func_adapter,
+                        &adapter_refs,
+                    )?;
+
+                    realm.to_js_value_facade(&val_adapter)
+                })
+            } else {
+                Ok(JsValueFacade::Null)
+            }
+        })
     }
     pub fn js_invoke_function_sync<R: JsRuntimeFacadeInner>(
         &self,
-        _rti: &R,
-        _args: Vec<JsValueFacade>,
+        rti: &R,
+        args: Vec<JsValueFacade>,
     ) -> Result<JsValueFacade, JsError> {
-        todo!()
+        self.cached_object.with_obj_sync(rti, |realm, func_adapter| {
+            //
+            let mut adapter_args = vec![];
+            for arg in args {
+                adapter_args.push(realm.from_js_value_facade(arg)?);
+            }
+
+            let adapter_refs: Vec<&<<<<R as JsRuntimeFacadeInner>::JsRuntimeFacadeType as JsRuntimeFacade>::JsRuntimeAdapterType as JsRuntimeAdapter>::JsRealmAdapterType as JsRealmAdapter>::JsValueAdapterType> = adapter_args.iter().collect();
+
+            let val_adapter = realm.js_function_invoke(
+                None,
+                func_adapter,
+                &adapter_refs,
+            )?;
+
+            realm.to_js_value_facade(&val_adapter)
+        })?
     }
 }
 
+pub enum TypedArrayType {
+    Uint8,
+}
+
 /// The JsValueFacade is a Send-able representation of a value in the Script engine
 #[allow(clippy::type_complexity)]
 pub enum JsValueFacade {
@@ -284,6 +342,15 @@ pub enum JsValueFacade {
     JsError {
         val: JsError,
     },
+    ProxyInstance {
+        namespace: &'static [&'static str],
+        class_name: &'static str,
+        instance_id: JsProxyInstanceId,
+    },
+    TypedArray {
+        buffer: Vec<u8>,
+        array_type: TypedArrayType,
+    },
     Null,
     Undefined,
 }
@@ -404,6 +471,8 @@ impl JsValueFacade {
             JsValueFacade::JsArray { .. } => JsValueType::Array,
             JsValueFacade::JsFunction { .. } => JsValueType::Function,
             JsValueFacade::JsError { .. } => JsValueType::Error,
+            JsValueFacade::ProxyInstance { .. } => JsValueType::Object,
+            JsValueFacade::TypedArray { .. } => JsValueType::Object,
         }
     }
     pub fn stringify(&self) -> String {
@@ -455,18 +524,96 @@ impl JsValueFacade {
             JsValueFacade::Null => "Null".to_string(),
             JsValueFacade::Undefined => "Undefined".to_string(),
             JsValueFacade::JsError { val } => format!("{}", val),
+            JsValueFacade::ProxyInstance { .. } => "ProxyInstance".to_string(),
+            JsValueFacade::TypedArray { .. } => "TypedArray".to_string(),
         }
     }
 }
 
+pub trait JsValueConvertable {
+    fn to_js_value_facade(self) -> JsValueFacade;
+}
+
+impl JsValueConvertable for bool {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::new_bool(self)
+    }
+}
+
+impl JsValueConvertable for i32 {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::new_i32(self)
+    }
+}
+
+impl JsValueConvertable for f64 {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::new_f64(self)
+    }
+}
+
+impl JsValueConvertable for &str {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::new_str(self)
+    }
+}
+
+impl JsValueConvertable for String {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::new_string(self)
+    }
+}
+
+impl JsValueConvertable for Vec<u8> {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::TypedArray {
+            buffer: self,
+            array_type: TypedArrayType::Uint8,
+        }
+    }
+}
+
+impl JsValueConvertable for Vec<JsValueFacade> {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::Array { val: self }
+    }
+}
+
+impl JsValueConvertable for HashMap<String, JsValueFacade> {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::Object { val: self }
+    }
+}
+/* todo
+impl JsValueConvertable for Fn(&[JsValueFacade]) -> Result<JsValueFacade, JsError> + Send + Sync {
+    fn to_js_value_facade(self) -> JsValueFacade {
+        JsValueFacade::Function {
+            name: "".to_string(),
+            arg_count: 0,
+            func: Arc::new(Box::new(self)),
+        }
+    }
+}
+ */
+
 #[cfg(test)]
 pub mod tests {
-    use crate::js_utils::facades::values::JsValueFacade;
+    use crate::js_utils::facades::values::{CachedJsFunctionRef, CachedJsObjectRef, JsValueFacade};
+    use std::sync::Mutex;
 
     #[test]
     fn test_jsvf() {
         fn test<A: Send + Sync>(_a: A) {}
 
         test(JsValueFacade::Null);
+        test(JsValueFacade::JsFunction {
+            cached_function: CachedJsFunctionRef {
+                cached_object: CachedJsObjectRef {
+                    id: 0,
+                    realm_id: "".to_string(),
+                    drop_action: Mutex::new(None),
+                },
+            },
+        })
     }
 }
