@@ -32,7 +32,7 @@ impl JsError {
     }
     pub fn new_string(err: String) -> Self {
         JsError {
-            name: "".to_string(),
+            name: "Error".to_string(),
             message: err,
             stack: "".to_string(),
         }
@@ -50,7 +50,7 @@ impl JsError {
 
 impl std::fmt::Display for JsError {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
-        let e = format!("{}: {} at{}", self.name, self.message, self.stack);
+        let e = format!("{}: {}\n{}", self.name, self.message, self.stack);
         f.write_str(e.as_str())
     }
 }
